@@ -1,10 +1,10 @@
 ;;; csv-mode.el --- Major mode for editing comma/char separated values  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2003, 2004, 2012-2019  Free Software Foundation, Inc
+;; Copyright (C) 2003, 2004, 2012-2020  Free Software Foundation, Inc
 
 ;; Author: "Francis J. Wright" <F.J.Wright@qmul.ac.uk>
 ;; Maintainer: emacs-devel@gnu.org
-;; Version: 1.10
+;; Version: 1.11
 ;; Package-Requires: ((emacs "24.1") (cl-lib "0.5"))
 ;; Keywords: convenience
 
@@ -273,6 +273,8 @@ Number of spaces used by `csv-align-mode' and `csv-align-fields' after separator
     (define-key map [(control ?c) (control ?r)] 'csv-reverse-region)
     (define-key map [(control ?c) (control ?n)] 'csv-sort-numeric-fields)
     (define-key map [(control ?c) (control ?s)] 'csv-sort-fields)
+    (define-key map "\t" #'csv-tab-command)
+    (define-key map [backtab] #'csv-backtab-command)
     map))
 
 ;;;###autoload
@@ -657,17 +659,17 @@ point or marker arguments, BEG and END, delimiting the region."
 ;;;  Moving by field
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defsubst csv-end-of-field ()
+(defun csv-end-of-field ()
   "Skip forward over one field."
   (skip-chars-forward " ")
-  (if (eq (char-syntax (following-char)) ?\")
+  (if (memq (char-syntax (following-char)) csv-field-quotes)
       (goto-char (scan-sexps (point) 1)))
   (skip-chars-forward csv--skip-chars))
 
-(defsubst csv-beginning-of-field ()
+(defun csv-beginning-of-field ()
   "Skip backward over one field."
   (skip-syntax-backward " ")
-  (if (eq (char-syntax (preceding-char)) ?\")
+  (if (memq (char-syntax (preceding-char)) csv-field-quotes)
       (goto-char (scan-sexps (point) -1)))
   (skip-chars-backward csv--skip-chars))
 
@@ -704,9 +706,25 @@ move forward across N fields."
       (if (memq (preceding-char) csv-separator-chars) (backward-char))
       (csv-beginning-of-field))))
 
+(defun csv-tab-command ()
+  "Skip to the next field on the same line.
+Create a new field at end of line, if needed."
+  (interactive)
+  (skip-chars-forward csv--skip-chars)
+  (if (eolp)
+      (insert (car csv-separators))
+    (forward-char 1)))
+
+(defun csv-backtab-command ()
+  "Skip to the beginning of the previous field."
+  (interactive)
+  (skip-chars-backward csv--skip-chars)
+  (forward-char -1)
+  (skip-chars-backward csv--skip-chars))
+
 (defun csv-sort-skip-fields (n &optional yank)
   "Position point at the beginning of field N on the current line.
-Fields are separated by `csv-separators'\; null terminal field allowed.
+Fields are separated by `csv-separators'; null terminal field allowed.
 Assumes point is initially at the beginning of the line.
 YANK non-nil allows N to be greater than the number of fields, in
 which case extend the record as necessary."
@@ -1713,236 +1731,6 @@ setting works better)."
   ;; of the form \n, \t, \r, and \\ instead.
   (setq-local csv-field-quotes nil))
 
-;;;; ChangeLog:
-
-;; 2019-10-22  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el (csv-align--cursor-truncated): Fix C-e
-;; 	case
-;; 
-;; 2019-10-22  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Auto-shorten columns as well
-;; 
-;; 	(csv--column-widths): Also return the position of the widest field in 
-;; 	each column.
-;; 	(csv-align-fields, csv--jit-align): Update accordingly.
-;; 	(csv--jit-width-change): New function.
-;; 	(csv--jit-merge-columns): Use it on overlays placed on the widest field 
-;; 	of each column, to detect when they're shortened.
-;; 
-;; 2019-10-19  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: More cvs-align-mode improvements
-;; 
-;; 	Rename csv-align-fields-* to cvs-align-*.
-;; 	(csv-transpose): Use split-string.
-;; 	(csv-split-string): Delete function.
-;; 	(csv--config-column-widths): New var.
-;; 	(csv-align--set-column): New function.
-;; 	(csv-align-set-column-width): New command.
-;; 	(csv--jit-align): Use them to obey the per-column width settings. Delay
-;; 	context refresh by jit-lock-context-time. Set cursor-sensor-functions to
-;; 	untruncate fields on-the-fly.
-;; 	(csv-align--cursor-truncated): New function.
-;; 	(csv-align-mode): Activate cursor-sensor-mode.
-;; 
-;; 2019-10-19  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Fix incorrect truncation
-;; 
-;; 	(csv--field-index): New function, extracted from csv-field-index.
-;; 	(csv--jit-align): Don't apply csv-align-fields-max-width to the last
-;; 	column.	 Fix move-to-column call.
-;; 
-;; 2019-10-10  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Fix header-line's alignment
-;; 
-;; 	(csv-header-line): Change csv--header-line into an overlay. Add a
-;; 	modification-hooks to auto-refresh the header-line.
-;; 	(csv--header-flush, csv--header-string): New functions.
-;; 	(csv--compute-header-string): Make sure jit-lock was applied. 
-;; 	csv--header-hscroll can be nil sometimes somehow!
-;; 	(csv--jit-flush, csv-align-fields-mode): Flush header-line as well.
-;; 	(csv--jit-align): Flush header-line when applicable.  Fix typo.
-;; 
-;; 2019-10-09  Filipp Gunbin  <fgunbin@fastmail.fm>
-;; 
-;; 	packages/csv-mode/csv-mode.el: Fix csv-align-fields doc
-;; 
-;; 	(csv-align-fields): docstring mentioned csv-align-fields instead of 
-;; 	csv-align-padding
-;; 
-;; 2019-09-29  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Remove Francis as maintainer
-;; 
-;; 	(csv-unalign-fields): Also remove the `invisible` property since we use
-;; 	it to truncate fields in csv--jit-align.
-;; 	(csv-align-fields-max-width): Rename from csv-align-field-max-width to 
-;; 	match the "csv-align-fields" prefix.
-;; 	(csv--ellipsis-width): New function.
-;; 	(csv--jit-align): Use it to truncate more correctly.
-;; 
-;; 2019-09-27  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el (csv-align-field-max-width): New var
-;; 
-;; 	(csv--jit-unalign): Erase invisible property as well.
-;; 	(csv--jit-align): Truncate field to fit within csv-align-field-max-width 
-;; 	when needed.
-;; 	(csv-align-fields-mode): Add/remove `csv-truncate` to invisibility spec.
-;; 
-;; 2019-09-27  Francis Wright  <f.j.wright@qmul.ac.uk>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Fix for customize-mode
-;; 
-;; 	(csv-mode, tsv-mode): Specify :group explicitly for `customize-mode`s
-;; 	benefit
-;; 
-;; 2019-09-24  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Add tsv-mode and csv-align-fields-mode
-;; 
-;; 	Require cl-lib. Don't set buffer-invisibility-spec directly.
-;; 	(csv--skip-chars): Rename from misleading csv--skip-regexp.
-;; 	(csv-mode): Set normal-auto-fill-function to really disable
-;; 	auto-fill-mode.
-;; 	(csv--column-widths): Only operate over new args beg..end.
-;; 	(csv-align-fields): No need to narrow before csv--column-widths any
-;; 	more.
-;; 	(csv-align-fields-mode): New minor mode.
-;; 	(tsv-mode): New major mode.
-;; 
-;; 2019-09-18  Simen Heggestøyl  <simenheg@gmail.com>
-;; 
-;; 	Speed up 'csv-align-fields'
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Bump version number and make the 
-;; 	dependency on Emacs 24.1 or higher explicit.
-;; 	(csv--column-widths): Return the field widths as well.
-;; 	(csv-align-fields): Speed up by using the field widths already computed 
-;; 	by 'csv--column-widths' (bug#37393).
-;; 
-;; 2017-12-05  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode/csv-mode.el (csv-header-line): New command
-;; 
-;; 	(csv-menu): Add an entry for it.
-;; 	(csv--header-line, csv--header-hscroll, csv--header-string): New vars.
-;; 	(csv--compute-header-string): New function.
-;; 
-;; 2016-07-11  Paul Eggert	 <eggert@cs.ucla.edu>
-;; 
-;; 	Fix some quoting problems in doc strings
-;; 
-;; 	Most of these are minor issues involving, e.g., quoting `like this' 
-;; 	instead of 'like this'.	 A few involve escaping ` and ' with a preceding
-;; 	\= when the characters should not be turned into curved single quotes.
-;; 
-;; 2016-04-21  Leo Liu  <sdl.web@gmail.com>
-;; 
-;; 	Fix csv-mode to delete its own overlays only
-;; 
-;; 	* csv-mode/csv-mode.el (csv--make-overlay, csv--delete-overlay): New
-;; 	 functions.
-;; 	 (csv-align-fields, csv-unalign-fields, csv-transpose): Use them.
-;; 
-;; 2016-03-04  Francis Wright  <f.j.wright@qmul.ac.uk>
-;; 
-;; 	* csv-mode/csv-mode.el: Remove out-of-date "URL:" header.
-;; 
-;; 2016-03-03  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode, landmark: Fix maintainer's email
-;; 
-;; 2015-07-09  Leo Liu  <sdl.web@gmail.com>
-;; 
-;; 	Fix column width calculation in cvs-mode.el
-;; 
-;; 	* csv-mode/cvs-mode.el (csv--column-widths, csv-align-fields): Fix
-;; 	 column width calculation.
-;; 
-;; 2015-05-24  Leo Liu  <sdl.web@gmail.com>
-;; 
-;; 	* csv-mode/cvs-mode.el (csv-set-comment-start): Handle nil.
-;; 
-;; 	See also http://debbugs.gnu.org/20564.
-;; 
-;; 2015-04-15  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	(csv-mode): Set mode-line-position rather than mode-line-format.
-;; 
-;; 	Fixes: debbugs:20343
-;; 
-;; 	* csv-mode/csv-mode.el (csv-mode-line-format): Only keep the CSV part of
-;; 	the mode line.
-;; 
-;; 2014-01-15  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode (csv-mode-line-help-echo): Remove.
-;; 
-;; 2013-04-24  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode.el (csv-kill-one-field): Check for presence before deleting
-;; 	trailing separator.  Remove last arg and turn into a function.
-;; 	(csv-kill-one-column, csv-kill-many-columns): Adjust callers.
-;; 
-;; 2012-10-22  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el (csv-end-of-field): Don't skip TABs.
-;; 	(csv--skip-regexp): Rename from csv-skip-regexp.
-;; 
-;; 2012-10-10  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode.el: Bump version number.
-;; 
-;; 2012-10-10  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* csv-mode.el: Use lexical-binding.  Remove redundant :group args.
-;; 	(csv-separators): Add TAB to the default.
-;; 	(csv-invisibility-default): Change default to t.
-;; 	(csv-separator-face): Inherit from escape-glyph.  Remove variable.
-;; 	(csv-mode-line-format): Remove trailing "--".  Move next to line-number.
-;; 	(csv-interactive-args): Use use-region-p.
-;; 	(csv--column-widths): New function, extracted from csv-align-fields.
-;; 	(csv-align-fields): Use it.  Use whole buffer by default. Use :align-to
-;; 	and text-properties when possible.
-;; 	(csv-unalign-fields): Also remove properties.
-;; 	(csv-mode): Truncate lines.
-;; 
-;; 2012-03-24  Chong Yidong  <cyd@gnu.org>
-;; 
-;; 	Commentary fix for quarter-plane.el.
-;; 
-;; 2012-03-24  Chong Yidong  <cyd@gnu.org>
-;; 
-;; 	Commentary tweaks for csv-mode, ioccur, and nhexl-mode packages.
-;; 
-;; 2012-03-24  Chong Yidong  <cyd@gnu.org>
-;; 
-;; 	csv-mode.el: Improve commentary.
-;; 
-;; 2012-03-12  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/csv-mode/csv-mode.el: Minor installation cleanups. Fix up
-;; 	copyright notice.  Set version.
-;; 	(csv-separators, csv-field-quotes): Fix calls to `error'.
-;; 	(csv-mode-line-help-echo, csv-mode-line-format): Replace
-;; 	mode-line-format for default-mode-line-format.
-;; 	(csv-mode-map): Declare and initialize.
-;; 	(csv-mode): Add autoload cookie.
-;; 	(csv-set-comment-start): Make sure vars are made buffer-local.
-;; 	(csv-field-index-mode, csv-field-index): Use derived-mode-p.
-;; 	(csv-align-fields): Improve insertion types of overlay's markers.
-;; 
-;; 2012-03-12  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	Add csv-mode.el.
-;; 
-
-
 
 (provide 'csv-mode)
 
